import { useDevicePixelRatio } from '@vueuse/core'
import {
  APICreate,
  APIListPaginated,
  APIRemove,
  APIRetrieve,
  APIUpdate,
  createExtendableAPI,
  useObjectFromStore,
} from '@rokoli/bnb/drf'
import { defineStore } from 'pinia'
import { computed, ComputedRef, MaybeRefOrGetter, toValue } from 'vue'

import { steeringAuthInit } from '@/stores/auth'
import { createSteeringURL } from '@/api'

export type Image = {
  id: number
  altText: string
  credits: string
  licenseId: number | null
  isUseExplicitlyGrantedByAuthor: boolean
  image: string
  file?: File
  ppoi: string
  width: number | null
  height: number | null
  url: string
}
export type NewImage = Omit<Image, 'id' | 'image' | 'file'> & { file: File }
type ReadonlyAttrs = 'id' | 'width' | 'height' | 'thumbnails' | 'file' | 'image'
type ImageCreateData = Omit<Image, ReadonlyAttrs> & { image: File }
type ImageUpdateData = Partial<Omit<Image, ReadonlyAttrs>>

const endpoint = createSteeringURL.prefix('images')

export const useImageStore = defineStore('images', () => {
  const { api, base } = createExtendableAPI<Image>(endpoint, steeringAuthInit)
  return {
    ...base,
    ...APIListPaginated(api),
    ...APIRetrieve(api),
    ...APICreate<Image, ImageCreateData | FormData>(api),
    ...APIUpdate<Image, ImageUpdateData | FormData, Partial<ImageUpdateData> | FormData>(api),
    ...APIRemove(api),
  }
})

export function useImage(
  imageId: MaybeRefOrGetter<number | null | undefined>,
): ComputedRef<Image | null> {
  const { obj } = useObjectFromStore(() => toValue(imageId) ?? null, useImageStore())
  return obj
}

type UseImageUrlOptions = {
  width?: MaybeRefOrGetter<number | null | undefined>
  height?: MaybeRefOrGetter<number | null | undefined>
}

export function useImageUrl(
  image: MaybeRefOrGetter<Image | Image['id'] | null | undefined>,
  options: UseImageUrlOptions | undefined = undefined,
) {
  const { pixelRatio } = useDevicePixelRatio()
  return computed(() => {
    const _image = toValue(image)
    if (!_image) return null
    const imageId = typeof _image === 'number' ? _image : _image.id

    const queryArgs = new URLSearchParams()
    const width = toValue(options?.width)
    const height = toValue(options?.height)
    const ratio = pixelRatio.value
    if (width) queryArgs.set('width', Math.round(toValue(width) * ratio).toString())
    if (height) queryArgs.set('height', Math.round(toValue(height) * ratio).toString())

    return endpoint(imageId, 'render', queryArgs)
  })
}
